#ifndef DYN_VALUE_HH
#define DYN_VALUE_HH

#include <awali/common/tuple.hh> // tuple ostream << operator
#include <awali/sttc/empty.hh> // empty ostream << operator
#include"untyped_value.hh"
#include<typeinfo>

namespace awali { namespace dyn {
  template<typename T>
  T any_cast(const any_t& a);
  namespace internal {

    template<typename T>
    struct Value : untyped_value {
      Value(const T& val) : val(val) {}
      T val;
      Value* clone() const {
	return new Value(val);
      }

      std::ostream& output(std::ostream& o) const{
	return (o << val);
      }

      bool less(const untyped_value& uv) const;

      bool equal(const untyped_value& uv) const{
	const Value& v=dynamic_cast<const Value&>(uv);
	return (val == v.val);
      }

      std::ostream&  real_type_name(std::ostream& o) const {
	o << typeid(val).name();
	return o;
      }
     
      virtual ~Value() {}
    };

    template<typename T>
    bool Value<T>::less(const untyped_value& uv) const {
      const Value& v=dynamic_cast<const Value&>(uv);
      return (val < v.val);
    }

    template<>
    bool Value<std::string>::less(const untyped_value& uv) const {
      const Value& v=dynamic_cast<const Value&>(uv);
      if(val.length() == v.val.length())
	return (val < v.val);
      else
	return (val.length() < v.val.length());
    }
  }//end of ns internal

}}//end of ns awali::dyn
#endif //DYN_VALUE_HH
